// DShowInputDriver: DirectShow based input plugin for VirtualDub
// Copyright (C) 2010-2011 Avery Lee
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#ifndef ALLOCATOR_H
#define ALLOCATOR_H

#include <vd2/system/vdtypes.h>
#include "Utils.h"

class MemMediaSample : public IMediaSample {
	MemMediaSample(const MemMediaSample&);
	MemMediaSample& operator=(const MemMediaSample&);
public:
	MemMediaSample();
	~MemMediaSample();

	HRESULT Init(IMemAllocator *pAlloc, const ALLOCATOR_PROPERTIES& props);
	bool Init(uint32 size);

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE GetPointer(BYTE **ppBuffer);
	long STDMETHODCALLTYPE GetSize();
	HRESULT STDMETHODCALLTYPE GetTime(REFERENCE_TIME *pTimeStart, REFERENCE_TIME *pTimeEnd);
	HRESULT STDMETHODCALLTYPE SetTime(REFERENCE_TIME *pTimeStart, REFERENCE_TIME *pTimeEnd);        
	HRESULT STDMETHODCALLTYPE IsSyncPoint();
	HRESULT STDMETHODCALLTYPE SetSyncPoint(BOOL bIsSyncPoint);
	HRESULT STDMETHODCALLTYPE IsPreroll();
	HRESULT STDMETHODCALLTYPE SetPreroll(BOOL bIsPreroll);
	long STDMETHODCALLTYPE GetActualDataLength();
	HRESULT STDMETHODCALLTYPE SetActualDataLength(long Length);
	HRESULT STDMETHODCALLTYPE GetMediaType(AM_MEDIA_TYPE **ppMediaType);
	HRESULT STDMETHODCALLTYPE SetMediaType(AM_MEDIA_TYPE *pMediaType);
	HRESULT STDMETHODCALLTYPE IsDiscontinuity();
	HRESULT STDMETHODCALLTYPE SetDiscontinuity(BOOL bDiscontinuity);
	HRESULT STDMETHODCALLTYPE GetMediaTime(LONGLONG *pTimeStart, LONGLONG *pTimeEnd);
	HRESULT STDMETHODCALLTYPE SetMediaTime(LONGLONG *pTimeStart, LONGLONG *pTimeEnd);

protected:
	LONG	mRefCount;
	IMemAllocator *mpAllocator;

	bool		mbIsPreroll;
	bool		mbIsSyncPoint;
	bool		mbIsDiscontinuity;
	bool		mbStreamStartTimeSet;
	bool		mbStreamStopTimeSet;
	bool		mbMediaTimesSet;
	long		mDataLength;
	void		*mpBuffer;
	void		*mpBufferBase;
	long		mBufferSize;
	LONGLONG	mMediaTimeStart;
	LONGLONG	mMediaTimeEnd;
	REFERENCE_TIME	mStreamTimeStart;
	REFERENCE_TIME	mStreamTimeEnd;
	MediaTypePtr	mpMediaType;
};

class MemAllocator : public IMemAllocator {
	MemAllocator(const MemAllocator&);
	MemAllocator& operator=(const MemAllocator&);
public:
	MemAllocator();
	~MemAllocator();

	HRESULT Init();

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE SetProperties(ALLOCATOR_PROPERTIES *pRequest, ALLOCATOR_PROPERTIES *pActual);
	HRESULT STDMETHODCALLTYPE GetProperties(ALLOCATOR_PROPERTIES *pProps);
	HRESULT STDMETHODCALLTYPE Commit();
	HRESULT STDMETHODCALLTYPE Decommit();
	HRESULT STDMETHODCALLTYPE GetBuffer(IMediaSample **ppBuffer, REFERENCE_TIME *pStartTime, REFERENCE_TIME *pEndTime, DWORD dwFlags);
	HRESULT STDMETHODCALLTYPE ReleaseBuffer(IMediaSample *pBuffer);

protected:
	Lock	mLock;
	HANDLE	mBufferAvailable;
	bool	mbCommitted;
	int		mWaiters;
	int		mBuffersOutstanding;

	std::vector<MemMediaSample *> mSamples;

	ALLOCATOR_PROPERTIES	mProps;

	LONG	mRefCount;
};

#endif
