// DShowInputDriver: DirectShow based input plugin for VirtualDub
// Copyright (C) 2010-2011 Avery Lee
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#ifndef AUDIOSOURCEDSHOW_H
#define AUDIOSOURCEDSHOW_H

#include <vd2/VDXFrame/Unknown.h>
#include <vd2/plugin/vdinputdriver.h>
#include <vd2/system/refcount.h>
#include <vd2/system/vdstl.h>
#include <vd2/system/VDString.h>
#include "Engine.h"
#include "SampleGrabber.h"

class DSAudioSource : public vdxunknown<IVDXAudioSource>, IVDXStreamSource, public IDSSampleGrabberCallback {
public:
	DSAudioSource(const VDXInputDriverContext *context, IDSEngine *engine, const wchar_t *fn);
	~DSAudioSource();

	void GetInfo(VDStringA& s);

	// IVDXUnknown
	int			VDXAPIENTRY AddRef();
	int			VDXAPIENTRY Release();
	void *		VDXAPIENTRY AsInterface(uint32 iid);

	// IVDXStreamSource
	void		VDXAPIENTRY GetStreamSourceInfo(VDXStreamSourceInfo&);

	bool		VDXAPIENTRY Read(sint64 lStart, uint32 lCount, void *lpBuffer, uint32 cbBuffer, uint32 *lBytesRead, uint32 *lSamplesRead);

	const void *VDXAPIENTRY GetDirectFormat();
	int			VDXAPIENTRY GetDirectFormatLen();

	ErrorMode	VDXAPIENTRY GetDecodeErrorMode();
	void		VDXAPIENTRY SetDecodeErrorMode(ErrorMode mode);
	bool		VDXAPIENTRY IsDecodeErrorModeSupported(ErrorMode mode);

	bool		VDXAPIENTRY IsVBR();
	sint64		VDXAPIENTRY TimeToPositionVBR(sint64 us);
	sint64		VDXAPIENTRY PositionToTimeVBR(sint64 samples);

	// IVDXAudioSource
	void		VDXAPIENTRY GetAudioSourceInfo(VDXAudioSourceInfo& info);

protected:
	void InitLocal(void *pvFn);
	void ShutdownLocal(void *);
	void DecodeLocal(void *pvReadCommandData);
	bool Process(IMediaSample *pSample, const REFERENCE_TIME& timeBase);
	void OnSampleMissed();

	vdrefptr<IDSEngine>			mpEngine;
	const VDXInputDriverContext *const mpContext;

	vdrefptr<IGraphBuilder>		mpGraph;
	vdrefptr<IMediaControl>		mpGraphMediaControl;
	vdrefptr<IMediaEvent>		mpGraphMediaEvent;
	vdrefptr<IMediaSeeking>		mpGraphMediaSeeking;
	vdrefptr<IMediaFilter>		mpGraphMediaFilter;
	vdrefptr<IPin>				mpSourcePin;
	vdrefptr<IPin>				mpRendererPin;
	vdrefptr<DSSampleGrabber>	mpSampleGrabber;
	vdrefptr<IBaseFilter>		mpRenderer;

	VDXStreamSourceInfo			mStreamInfo;
	VDXAudioSourceInfo			mAudioInfo;

	VDSignal	mCommandSignal;

	vdstructex<WAVEFORMATEX>	mFormat;

	// current read information
	REFERENCE_TIME mReadTimeStart;
	sint64	mReadPosition;
	uint32	mReadSampleCount;
	uint32	mReadBytesLeft;
	uint32	mReadBytesActual;
	void	*mpReadBuffer;
	bool	mbReadFirstSample;
	bool	mbSampleLate;

	sint64	mSeekFudgeOffset;

	uint32	mRegisteredGraphId;
	VDStringA	mInfo;
};

#endif
