// DShowInputDriver: DirectShow based input plugin for VirtualDub
// Copyright (C) 2010-2011 Avery Lee
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#ifndef BASEFILTER_H
#define BASEFILTER_H

#include "Utils.h"
#include "BasePin.h"

class BaseFilter : public IBaseFilter {
	BaseFilter(const BaseFilter&);
	BaseFilter& operator=(const BaseFilter&);

	friend class BaseFilterPinEnum;
public:
	BaseFilter();
	~BaseFilter();

	void ReservePins(unsigned pins);
	void AddPin(BasePin *pin, LPCWSTR id);

	virtual bool IsStopped();

	IFilterGraph *GetGraph() { return mpGraph; }
	BSTR GetNameInGraph() { return mpNameInGraph; }

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE EnumPins(IEnumPins **ppEnum);
	HRESULT STDMETHODCALLTYPE FindPin(LPCWSTR Id, IPin **ppPin);
	HRESULT STDMETHODCALLTYPE QueryFilterInfo(FILTER_INFO *pInfo);
	HRESULT STDMETHODCALLTYPE JoinFilterGraph(IFilterGraph *pGraph, LPCWSTR pName);
	HRESULT STDMETHODCALLTYPE QueryVendorInfo(LPWSTR *pVendorInfo);

	HRESULT STDMETHODCALLTYPE Stop();
	HRESULT STDMETHODCALLTYPE Pause();
	HRESULT STDMETHODCALLTYPE Run(REFERENCE_TIME tStart);
	HRESULT STDMETHODCALLTYPE GetState(DWORD dwMilliSecsTimeout, FILTER_STATE *State);
	HRESULT STDMETHODCALLTYPE SetSyncSource(IReferenceClock *pClock);
	HRESULT STDMETHODCALLTYPE GetSyncSource(IReferenceClock **pClock);

protected:
	IFilterGraph *mpGraph;
	BSTR mpNameInGraph;
	LONG mRefCount;

	typedef std::vector<BasePin *> PinSet;
	PinSet mPinSet;
	int	mPinSetGeneration;

	Lock mLock;
	IReferenceClock *mpClock;
	FILTER_STATE mState;
};

#endif
