// DShowInputDriver: DirectShow based input plugin for VirtualDub
// Copyright (C) 2010-2011 Avery Lee
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#ifndef BASEPIN_H
#define BASEPIN_H

#include "Utils.h"

class BaseFilter;

class BasePin : public IPin {
	BasePin(const BasePin&);
	BasePin& operator=(const BasePin&);
public:
	BasePin();
	~BasePin();

	LPCWSTR GetId() const { return mpId; }
	IPin *GetConnectedPin() const { return mpConnectedPin; }
	const AM_MEDIA_TYPE *GetConnectedMediaType() const { return mpConnectedMediaType; }

	void SetPinInfo(BaseFilter *pFilter, LPCWSTR id);

	virtual HRESULT Activate();
	virtual void Deactivate();

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE Disconnect();
	HRESULT STDMETHODCALLTYPE ConnectedTo(IPin **pPin);
	HRESULT STDMETHODCALLTYPE ConnectionMediaType(AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryPinInfo(PIN_INFO *pInfo);
	HRESULT STDMETHODCALLTYPE QueryId(LPWSTR *Id);
	HRESULT STDMETHODCALLTYPE EnumMediaTypes(IEnumMediaTypes **ppEnum);
	HRESULT STDMETHODCALLTYPE QueryInternalConnections(IPin **apPin, ULONG *nPin);
	HRESULT STDMETHODCALLTYPE NewSegment(REFERENCE_TIME tStart, REFERENCE_TIME tStop, double dRate);

protected:
	LPCWSTR mpId;

	IPin *mpConnectedPin;
	MediaTypePtr mpConnectedMediaType;
	BaseFilter *mpFilter;

	Lock mLock;
};

class InputPin : public BasePin {
public:
	InputPin();
	~InputPin();

	HRESULT STDMETHODCALLTYPE Connect(IPin *pReceivePin, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE ReceiveConnection(IPin *pConnector, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryAccept(const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE BeginFlush();
	HRESULT STDMETHODCALLTYPE EndFlush();
	HRESULT STDMETHODCALLTYPE EndOfStream();
	HRESULT STDMETHODCALLTYPE QueryDirection(PIN_DIRECTION *pPinDir);
};

class OutputPin : public BasePin {
public:
	OutputPin();
	~OutputPin();

	HRESULT STDMETHODCALLTYPE Connect(IPin *pReceivePin, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE ReceiveConnection(IPin *pConnector, const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE QueryAccept(const AM_MEDIA_TYPE *pmt);
	HRESULT STDMETHODCALLTYPE BeginFlush();
	HRESULT STDMETHODCALLTYPE EndFlush();
	HRESULT STDMETHODCALLTYPE EndOfStream();
	HRESULT STDMETHODCALLTYPE QueryDirection(PIN_DIRECTION *pPinDir);
};

#endif
