// DShowInputDriver: DirectShow based input plugin for VirtualDub
// Copyright (C) 2010-2011 Avery Lee
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#ifndef MEMPIN_H
#define MEMPIN_H

#include "BasePin.h"

class MemInputPin : public InputPin, public IMemInputPin {
public:
	MemInputPin();
	~MemInputPin();

	void Deactivate();

	ULONG STDMETHODCALLTYPE AddRef();
	ULONG STDMETHODCALLTYPE Release();
	HRESULT STDMETHODCALLTYPE QueryInterface(REFIID riid, void **ppvObject);

	HRESULT STDMETHODCALLTYPE GetAllocator(IMemAllocator **ppAllocator);
	HRESULT STDMETHODCALLTYPE NotifyAllocator(IMemAllocator *pAllocator, BOOL bReadOnly);
	HRESULT STDMETHODCALLTYPE GetAllocatorRequirements(ALLOCATOR_PROPERTIES *pProps);
	HRESULT STDMETHODCALLTYPE Receive(IMediaSample *pSample);

	HRESULT STDMETHODCALLTYPE ReceiveMultiple(IMediaSample **pSamples, long nSamples, long *nSamplesProcessed);
	HRESULT STDMETHODCALLTYPE ReceiveCanBlock();

protected:
	IMemAllocator *mpAllocator;
};

class MemOutputPin : public OutputPin {
public:
	MemOutputPin();
	~MemOutputPin();

	HRESULT STDMETHODCALLTYPE Disconnect();
	HRESULT STDMETHODCALLTYPE Connect(IPin *pReceivePin, const AM_MEDIA_TYPE *pmt);

protected:
	IMemAllocator *mpAllocator;
	IMemInputPin *mpTransportPin;
};

#endif
