// DShowInputDriver: DirectShow based input plugin for VirtualDub
// Copyright (C) 2010-2011 Avery Lee
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

#include "stdafx.h"
#include <vd2/system/registry.h>
#include <vd2/system/strutil.h>
#include <vd2/system/vdtypes.h>
#include <vd2/system/w32assist.h>
#include "resource.h"
#include <windows.h>
#include <commctrl.h>

void ShowUIDialogConfig(VDGUIHandle hwnd);
void ShowUIDialogAbout(VDGUIHandle hwnd);

class UIDialogInfo {
public:
	UIDialogInfo(const char *s) : mpText(s) {}

	void ShowDialog(HWND hwndParent);

protected:
	static INT_PTR CALLBACK StaticDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam);
	INT_PTR DlgProc(UINT msg, WPARAM wParam, LPARAM lParam);

	bool OnInitDialog();
	bool OnCommand(uint32 id);

	HWND mhdlg;
	HWND mhwndList;

	const char *mpText;
};

void UIDialogInfo::ShowDialog(HWND hwndParent) {
	HINSTANCE hInst = VDGetLocalModuleHandleW32();

	DialogBoxParam(hInst, MAKEINTRESOURCE(IDD_INFO), hwndParent, StaticDlgProc, (LPARAM)this);
}

INT_PTR CALLBACK UIDialogInfo::StaticDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam) {
	UIDialogInfo *p;

	if (msg == WM_INITDIALOG) {
		SetWindowLongPtr(hdlg, DWLP_USER, (LONG_PTR)lParam);

		p = (UIDialogInfo *)lParam;
		p->mhdlg = hdlg;
	} else {
		p = (UIDialogInfo *)GetWindowLongPtr(hdlg, DWLP_USER);
	}

	return p ? p->DlgProc(msg, wParam, lParam) : FALSE;
}

INT_PTR UIDialogInfo::DlgProc(UINT msg, WPARAM wParam, LPARAM lParam) {
	switch(msg) {
		case WM_INITDIALOG:
			return OnInitDialog();

		case WM_COMMAND:
			return OnCommand(LOWORD(wParam));
	}

	return FALSE;
}

bool UIDialogInfo::OnInitDialog() {
	mhwndList = GetDlgItem(mhdlg, IDC_LIST);

	if (mhwndList) {
		LVCOLUMN col = {};
		col.mask = LVCF_TEXT | LVCF_WIDTH;
		col.cx = 50;
		col.pszText = "";

		ListView_InsertColumn(mhwndList, 0, &col);
		ListView_InsertColumn(mhwndList, 1, &col);

		SendMessage(mhwndList, WM_SETREDRAW, FALSE, 0);

		VDStringA str;
		const char *s = mpText;

		int lines = 0;
		for(;;) {
			const char *split = s;

			while(*split && *split != '\t' && *split != '\n')
				++split;

			const char *split2 = split;
			if (*split2 == '\t')
				++split2;

			const char *eol = split2;

			while(*eol && *eol != '\n')
				++eol;

			if (*eol && !eol[1])
				break;

			LVITEM lvi = {0};
			lvi.mask = LVIF_TEXT;
			lvi.iItem = lines;
			lvi.iSubItem = 0;

			str.assign(s, split);
			lvi.pszText = (LPSTR)str.c_str();
			SendMessage(mhwndList, LVM_INSERTITEM, 0, (LPARAM)&lvi);

			if (split2 != eol) {
				str.assign(split2, eol);
				lvi.iSubItem = 1;
				lvi.pszText = (LPSTR)str.c_str();
				SendMessage(mhwndList, LVM_SETITEM, 0, (LPARAM)&lvi);
			}

			if (!*eol)
				break;

			++lines;
			s = eol + 1;
		}

		ListView_SetColumnWidth(mhwndList, 0, LVSCW_AUTOSIZE);
		ListView_SetColumnWidth(mhwndList, 1, LVSCW_AUTOSIZE);

		SendMessage(mhwndList, WM_SETREDRAW, TRUE, 0);
		InvalidateRect(mhwndList, NULL, TRUE);
	}

	SetFocus(GetDlgItem(mhdlg, IDC_LIST));
	return true;
}

bool UIDialogInfo::OnCommand(uint32 id) {
	switch(id) {
		case IDOK:
			EndDialog(mhdlg, TRUE);
			return true;

		case IDCANCEL:
			EndDialog(mhdlg, FALSE);
			return true;

		case IDC_CONFIG:
			ShowUIDialogConfig((VDGUIHandle)mhdlg);
			return true;

		case IDC_ABOUT:
			ShowUIDialogAbout((VDGUIHandle)mhdlg);
			return true;
	}

	return false;
}

///////////////////////////////////////////////////////////////////////////

void ShowUIDialogInfo(VDGUIHandle hwnd, const char *s) {
	UIDialogInfo dlg(s);

	dlg.ShowDialog((HWND)hwnd);
}
